--	=========================================================================
--	Replicate On Path Lua Module for Strata CX 5
--
--	MODULE		Replicate On Path
--	VERSION		1.0
--	AUTHOR		Jon Bradley
--	DATE		11.06.2006
--	
--	DESCRIPTION
--	
--	This module is to replicate an object a certain number of times if
--  the object has a motion path. This module will only work on objects
--  that already have animation applied to them.
--  
--  The script will create a new scripting menu entry under "Scripting"
--  called "Replicate On Path".
--
--  INSTALLATION
--  This script must be placed in one of the following paths:
--  
--  Main Application Path:
--  Strata 3D CX/Scripts/Autoload
--  
--  OR User Path (MAC)
--  User Name/Library/Application Support/Strata 3D CX/Scripts/Autoload
--
--  OR User Path (PC)
--  C:\Documents and Settings\USERNAME\Application Data\Strata 3D CX\Scripts\Autoload
--
--  USAGE
--  
--  To use this script, select the objects you want to replicate, and then
--  select the "Replication on Path" menu under the main "Scripting" Menu.
--
--  This script assumes that the object you are replicating has the animation
--  starting at time 0 (first keyframe at time 0).
--
--  The script determines the time of the last keyframe for the animation
--  object and replicates the object based on a time step - the total time
--  divided by the number of copies to create.
--
--  There are no settings to equally space the object along an animated path.
--  
--  You can have the script replicate the selected object so that each copy
--  is located on each animation path time marker (small squares on the
--  motion path of the selected object in your view). To do this, you just
--  need to choose a number of copies equal to 1 less than the frame of your
--  last keyframe. If your last keyframe is on frame 32, then 31 copies will
--  place the instances of your object on those visible markers.
--
--  Custom In/Out velocity changes are not supported in this script.
--  
--	LICENSE
--  
--	This work is licensed under the Creative Commons Attribution 2.5 License.
--	http://creativecommons.org/licenses/by/2.5/legalcode
--	=========================================================================

module("ReplicateOnPathModule", package.seeall)

function getInfo ()
	local c = s3d.database.GetActiveContext()
	local m = c and c:GetManipulatorManager()
	local size = m and m:GetSize()
	return c, m, size
end

function ReplicateOnPath(num)
	local c, m, size = getInfo()
	if size > 0 then

        --  By using "Clone()" on an instance our selection
        --  list is automatically changed each time we add
        --  a new copy of an object. To ensure we iterate
        --  our original list of selected objects, we
        --  create a temporary container to add replicated
        --  instances to the database all at once

        local aInstances = {}
        for s=1,size do
            aInstances[s] = m:GetIndexedInstance(s)
        end

        for s=1,size  do
        
            local i = aInstances[s]
            
            --  Get the time for the last keyframe (path animated)
            local endTime = i:GetLastKeyTime()

            if endTime then
                for t=1,num do
                    local timeStep = endTime/num
                    
                    -- Current time we are sampling at
                    local cTime = timeStep*t

                    print(string.format("Instance created at time: '%s'", cTime) )

                    --  Clone the instance in question
                    local inst = i:Clone()
                    
                    --  Here's a nice trick. Rather than having to 'explicitly'
                    --  deal with transformation order. We just set a keyframe
                    --  at cTime, then copy it to time 0, then delete all keyframes.
                    inst:ForceKeyTime(cTime)
                    
                    --  Move the current keyframe to time 0
                    inst:MoveKeyTimes(cTime,cTime,-cTime)
                    
                    --  Delete all keyframes of the cloned instance
                    inst:DeleteKeyTimes(0,endTime)
                    
                    c:AddInstance(inst)
                end
            end
            c:Refresh()
        end
	end
end

--	Replicate on Path UI Dialog
function ReplicateOnPathUI()
	local d = s3d.ui.Dialog("Replication On Path")
	d:BeginCluster("Settings:")
	d:AddStaticText("Number of Copies")
	local num = d:AddNumberText(0,2,40)
	d:EndCluster()
	d:SetLabel(s3d.ui.Dialog.kOkay, "Replicate")
	d:SetLabel(s3d.ui.Dialog.kCancel, "Cancel")
	local function checkUI(id)
        if d:GetValue(num) > 0 then
    		if id == s3d.ui.Dialog.kOkay then
                ReplicateOnPath( d:GetValue(num) )
    		end
        else
            s3d.ui.Alert ("Number of copies must be greater than 0")
        end
	end
	d:Present( checkUI )
end


local function GetActiveSelectionSize ()
	local c = s3d.database.GetActiveContext()
	local m = c and c:GetManipulatorManager()
	return m and m:GetSize()
end

local function EnableWithSelection (item)
	local size = GetActiveSelectionSize()
	return size and size > 0
end

local separator = { "-" }
local replicateOnPathMenuItem = { "Replicate on Path", 
        function (menuItem)
            ReplicateOnPathUI()
        end,
        EnableWithSelection
    }

-- Insert this menu item (and a separator) into the Scripting Menu
table.insert(s3d.ui.scripting_menu, separator)
table.insert(s3d.ui.scripting_menu, replicateOnPathMenuItem)